@@ -21,12 +21,14 @@
 #include "dlg_specific.h"
 #include "misc.h"
 
-#include "convert.h"
-
-#include "multibyte.h"
 #include "pgapifunc.h"
 
-extern GLOBAL_VALUES globals;
+GLOBAL_VALUES globals;
+
+/*
+ *	TRUE means that even though drivername is different, share common driver info.
+ */
+static BOOL share_common_driver_info = TRUE;
 
 static void encode(const pgNAME, char *out, int outlen);
 static pgNAME decode(const char *in);
@@ -220,6 +222,47 @@ makeKeepaliveConnectString(char *target, const ConnInfo *ci, BOOL abbrev)
 	return target;
 }
 
+#define OPENING_BRACKET '{'
+#define CLOSING_BRACKET '}'
+static const char *
+makePqoptConnectString(char **target, const ConnInfo *ci, BOOL abbrev)
+{
+	const char	*istr, *iptr;
+	char	*buf, *optr;
+	int	len;
+
+	istr = SAFE_NAME(ci->pqopt);
+	if (!istr[0])
+		return NULL_STRING;
+
+	for (iptr = istr, len = 0; *iptr; iptr++)
+	{
+		if (CLOSING_BRACKET == *iptr)
+			len++;
+		len++;
+	}
+	len += 30;
+	if (buf = (char *) malloc(len), buf == NULL)
+		return NULL_STRING;
+	if (abbrev)
+		snprintf(buf, len, ABBR_PQOPT "=%c", OPENING_BRACKET);
+	else
+		snprintf(buf, len, INI_PQOPT "=%c", OPENING_BRACKET);
+	optr = strchr(buf, '\0');
+	for (iptr = istr; *iptr; iptr++)
+	{
+		if (CLOSING_BRACKET == *iptr)
+			*(optr++) = *iptr;
+		*(optr++) = *iptr;
+	}
+	*(optr++) = CLOSING_BRACKET;
+	*(optr++) = ';';
+	*optr = '\0';
+	*target = buf;
+
+	return buf;
+}
+
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 char *
 makeXaOptConnectString(char *target, const ConnInfo *ci, BOOL abbrev)
@@ -246,6 +289,7 @@ makeConnectString(char *connect_string, const ConnInfo *ci, UWORD len)
 {
 	char		got_dsn = (ci->dsn[0] != '\0');
 	char		encoded_item[LARGE_REGISTRY_LEN];
+	char		*pqoptStr = NULL;
 	char		keepaliveStr[64];
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	char		xaOptStr[16];
@@ -315,7 +359,8 @@ inolog("hlen=%d", hlen);
 			INI_BYTEAASLONGVARBINARY "=%d;"
 			INI_USESERVERSIDEPREPARE "=%d;"
 			INI_LOWERCASEIDENTIFIER "=%d;"
-			"%s"
+			"%s"		/* INI_PQOPT */
+			"%s"		/* INIKEEPALIVE TIME/INTERVAL */
 #ifdef	WIN32
 			INI_GSSAUTHUSEGSSAPI "=%d;"
 #endif /* WIN32 */
@@ -349,6 +394,7 @@ inolog("hlen=%d", hlen);
 			,ci->bytea_as_longvarbinary
 			,ci->use_server_side_prepare
 			,ci->lower_case_identifier
+			,makePqoptConnectString(&pqoptStr, ci, FALSE)
 			,makeKeepaliveConnectString(keepaliveStr, ci, FALSE)
 #ifdef	WIN32
 			,ci->gssauth_use_gssapi
@@ -428,7 +474,8 @@ inolog("hlen=%d", hlen);
 				ABBR_MAXLONGVARCHARSIZE "=%d;"
 				INI_INT8AS "=%d;"
 				ABBR_EXTRASYSTABLEPREFIXES "=%s;"
-				"%s"
+				"%s"		/* ABBR_PQOPT */
+				"%s"		/* ABBRKEEPALIVE TIME/INTERVAL */
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 				"%s"
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
@@ -439,6 +486,7 @@ inolog("hlen=%d", hlen);
 				ci->drivers.max_longvarchar_size,
 				ci->int8_as,
 				ci->drivers.extra_systable_prefixes,
+				makePqoptConnectString(&pqoptStr, ci, TRUE),
 				makeKeepaliveConnectString(keepaliveStr, ci, TRUE),
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 				makeXaOptConnectString(xaOptStr, ci, TRUE),
@@ -475,6 +523,9 @@ inolog("hlen=%d", hlen);
 	}
 	if (olen < 0 || olen >= nlen) /* failed */
 		connect_string[0] = '\0';
+
+	if (NULL != pqoptStr)
+		free(pqoptStr);
 }
 
 static void
@@ -608,6 +659,10 @@ copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
 		else
 			ci->conn_settings = decode(value);
 	}
+	else if (stricmp(attribute, INI_PQOPT) == 0 || stricmp(attribute, ABBR_PQOPT) == 0)
+	{
+		ci->pqopt = decode_or_remove_braces(value);
+	}
 	else if (stricmp(attribute, INI_UPDATABLECURSORS) == 0 || stricmp(attribute, ABBR_UPDATABLECURSORS) == 0)
 		ci->allow_keyset = atoi(value);
 	else if (stricmp(attribute, INI_LFCONVERSION) == 0 || stricmp(attribute, ABBR_LFCONVERSION) == 0)
@@ -855,9 +910,18 @@ getDSNinfo(ConnInfo *ci, char overwrite)
 
 	if (ci->drivername[0] == '\0' || overwrite)
 	{
+		const char *drivername;
+
 		getDriverNameFromDSN(DSN, ci->drivername, sizeof(ci->drivername));
-		if (ci->drivername[0] && stricmp(ci->drivername, SAFE_NAME(ci->drivers.drivername)))
-			getCommonDefaults(ci->drivername, ODBCINST_INI, ci);
+		drivername = ci->drivername;
+		if (!share_common_driver_info &&
+		    drivername[0] &&
+		    stricmp(drivername, SAFE_NAME(ci->drivers.drivername)))
+		{
+			mylog("driver is about to change from '%s' to '%s'\n", SAFE_NAME(ci->drivers.drivername), drivername);
+			getCommonDefaults(drivername, ODBCINST_INI, ci);
+			getCommonDefaults(drivername, ODBCINST_INI, NULL);
+		}
 	}
 
 	/* Proceed with getting info for the given DSN. */
@@ -919,6 +983,11 @@ getDSNinfo(ConnInfo *ci, char overwrite)
 		SQLGetPrivateProfileString(DSN, INI_CONNSETTINGS, "", encoded_item, sizeof(encoded_item), ODBC_INI);
 		ci->conn_settings = decode(encoded_item);
 	}
+	if (NAME_IS_NULL(ci->pqopt))
+	{
+		SQLGetPrivateProfileString(DSN, INI_PQOPT, "", encoded_item, sizeof(encoded_item), ODBC_INI);
+		STRX_TO_NAME(ci->pqopt, encoded_item);
+	}
 
 	if (ci->translation_dll[0] == '\0' || overwrite)
 		SQLGetPrivateProfileString(DSN, INI_TRANSLATIONDLL, "", ci->translation_dll, sizeof(ci->translation_dll), ODBC_INI);
@@ -1039,12 +1108,14 @@ getDSNinfo(ConnInfo *ci, char overwrite)
 
 	if (get_qlog())
 	{
+#ifdef	NOT_USED
 		char	*enc = (char *) check_client_encoding(ci->conn_settings);
 
 		qlog("          conn_settings='%s', conn_encoding='%s'\n", ci->conn_settings,
 			NULL != enc ? enc : "(null)");
 		if (NULL != enc)
 			free(enc);
+#endif /* NOT_USED */
 		qlog("          translation_dll='%s',translation_option='%s'\n",
 			ci->translation_dll,
 			ci->translation_option);
@@ -1061,8 +1132,13 @@ writeDriverCommoninfo(const char *fileName, const char *sectionName,
 	char		tmp[128];
 	int		errc = 0;
 
-	if (stricmp(ODBCINST_INI, fileName) == 0 && NULL == sectionName)
-		sectionName = DBMS_NAME;
+	if (stricmp(ODBCINST_INI, fileName) == 0)
+	{
+		if (share_common_driver_info)
+			sectionName = DBMS_NAME;
+		else if (NULL == sectionName)
+			sectionName = DBMS_NAME;
+	}
 
 	sprintf(tmp, "%d", comval->commlog);
 	if (!SQLWritePrivateProfileString(sectionName, INI_COMMLOG, tmp, fileName))
@@ -1072,10 +1148,6 @@ writeDriverCommoninfo(const char *fileName, const char *sectionName,
 	if (!SQLWritePrivateProfileString(sectionName, INI_DEBUG, tmp, fileName))
 		errc--;
 
-	sprintf(tmp, "%d", comval->fetch_max);
-	if (!SQLWritePrivateProfileString(sectionName, INI_FETCH, tmp, fileName))
-		errc--;
-
 	if (stricmp(ODBCINST_INI, fileName) == 0)
 		return errc;
 
@@ -1212,6 +1284,11 @@ writeDSNinfo(const ConnInfo *ci)
 								 encoded_item,
 								 ODBC_INI);
 
+	SQLWritePrivateProfileString(DSN,
+								 INI_PQOPT,
+								 SAFE_NAME(ci->pqopt),
+								 ODBC_INI);
+
 	sprintf(temp, "%d", ci->allow_keyset);
 	SQLWritePrivateProfileString(DSN,
 								 INI_UPDATABLECURSORS,
@@ -1296,6 +1373,45 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 		comval = &(ci->drivers);
 	else
 		comval = &globals;
+
+	/*
+	 * globals.debug or globals.commlog means whether take mylog or commlog
+	 * out of the connection time or not but doesn't mean the default of
+	 * ci->drivers.debug(commlog).
+	 */
+	/* Debug is stored in the driver section */
+	SQLGetPrivateProfileString(section, INI_DEBUG, "",
+				   temp, sizeof(temp), filename);
+	if (temp[0])
+	{
+		if (inst_position && ci)
+			;
+		else
+			comval->debug = atoi(temp);
+	}
+	else if (inst_position)
+		comval->debug = DEFAULT_DEBUG;
+
+	/* CommLog is stored in the driver section */
+	SQLGetPrivateProfileString(section, INI_COMMLOG, "",
+				   temp, sizeof(temp), filename);
+	if (temp[0])
+	{
+		if (inst_position && ci)
+			;
+		else
+			comval->commlog = atoi(temp);
+	}
+	else if (inst_position)
+		comval->commlog = DEFAULT_COMMLOG;
+
+	if (!ci)
+		logs_on_off(0, 0, 0);
+
+	/*
+	 * If inst_position of xxxxxx is present(usually not present),
+	 * it is the default of ci->drivers.xxxxxx .
+	 */
 	/* Fetch Count is stored in driver section */
 	SQLGetPrivateProfileString(section, INI_FETCH, "",
 							   temp, sizeof(temp), filename);
@@ -1309,25 +1425,6 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 	else if (inst_position)
 		comval->fetch_max = FETCH_MAX;
 
-	/* Debug is stored in the driver section */
-	SQLGetPrivateProfileString(section, INI_DEBUG, "",
-							   temp, sizeof(temp), filename);
-	if (temp[0])
-		comval->debug = atoi(temp);
-	else if (inst_position)
-		comval->debug = DEFAULT_DEBUG;
-
-	/* CommLog is stored in the driver section */
-	SQLGetPrivateProfileString(section, INI_COMMLOG, "",
-							   temp, sizeof(temp), filename);
-	if (temp[0])
-		comval->commlog = atoi(temp);
-	else if (inst_position)
-		comval->commlog = DEFAULT_COMMLOG;
-
-	if (!ci)
-		logs_on_off(0, 0, 0);
-
 	/* Recognize Unique Index is stored in the driver section only */
 	SQLGetPrivateProfileString(section, INI_UNIQUEINDEX, "",
 							   temp, sizeof(temp), filename);
@@ -1563,15 +1660,29 @@ decode(const char *in)
 static pgNAME
 decode_or_remove_braces(const char *in)
 {
-	if ('{' == in[0])
+	if (OPENING_BRACKET == in[0])
 	{
 		size_t inlen = strlen(in);
-		if ('}' == in[inlen - 1]) /* enclosed by braces */
+		if (CLOSING_BRACKET == in[inlen - 1]) /* enclosed with braces */
 		{
+			int	i;
+			const char	*istr, *eptr;
+			char	*ostr;
 			pgNAME	out;
 
 			INIT_NAME(out);
-			STRN_TO_NAME(out, in + 1, inlen - 2);
+			if (NULL == (ostr = (char *) malloc(inlen)))
+				return out;
+			eptr = in + inlen - 1;
+			for (istr = in + 1, i = 0; *istr && istr < eptr; i++)
+			{
+				if (CLOSING_BRACKET == istr[0] &&
+				    CLOSING_BRACKET == istr[1])
+					istr++;
+				ostr[i] = *(istr++);
+			}
+			ostr[i] = '\0';
+			SET_NAME_DIRECTLY(out, ostr);
 			return out;
 		}
 	}
@@ -1687,3 +1798,142 @@ char *extract_extra_attribute_setting(const pgNAME setting, const char *attr)
 	mylog("extracted a %s '%s' from %s\n", attr, rptr, str);
 	return rptr;
 }
+
+void
+CC_conninfo_release(ConnInfo *conninfo)
+{
+	NULL_THE_NAME(conninfo->password);
+	NULL_THE_NAME(conninfo->conn_settings);
+	NULL_THE_NAME(conninfo->pqopt);
+	finalize_globals(&conninfo->drivers);
+}
+
+void
+CC_conninfo_init(ConnInfo *conninfo, UInt4 option)
+{
+	CSTR	func = "CC_conninfo_init";
+	mylog("%s opt=%d\n", func, option);
+
+	if (0 != (CLEANUP_FOR_REUSE & option))
+		CC_conninfo_release(conninfo);
+	memset(conninfo, 0, sizeof(ConnInfo));
+	conninfo->allow_keyset = -1;
+	conninfo->lf_conversion = -1;
+	conninfo->true_is_minus1 = -1;
+	conninfo->int8_as = -101;
+	conninfo->bytea_as_longvarbinary = -1;
+	conninfo->use_server_side_prepare = -1;
+	conninfo->lower_case_identifier = -1;
+	conninfo->rollback_on_error = -1;
+	conninfo->force_abbrev_connstr = -1;
+	conninfo->bde_environment = -1;
+	conninfo->fake_mss = -1;
+	conninfo->cvt_null_date_string = -1;
+	conninfo->accessible_only = -1;
+	conninfo->ignore_round_trip_time = -1;
+	conninfo->disable_keepalive = -1;
+	conninfo->gssauth_use_gssapi = -1;
+	conninfo->keepalive_idle = -1;
+	conninfo->keepalive_interval = -1;
+#ifdef	_HANDLE_ENLIST_IN_DTC_
+	conninfo->xa_opt = -1;
+#endif /* _HANDLE_ENLIST_IN_DTC_ */
+	if (0 != (COPY_GLOBALS & option))
+		copy_globals(&(conninfo->drivers), &globals);
+}
+
+#define	CORR_STRCPY(item)	strncpy_null(to->item, from->item, sizeof(to->item))
+#define	CORR_VALCPY(item)	(to->item = from->item)
+
+void	copy_globals(GLOBAL_VALUES *to, const GLOBAL_VALUES *from)
+{
+	memset(to, 0, sizeof(*to));
+	/***
+	memcpy(to, from, sizeof(GLOBAL_VALUES));
+	SET_NAME_DIRECTLY(to->drivername, NULL);
+	SET_NAME_DIRECTLY(to->conn_settings, NULL);
+	***/
+	NAME_TO_NAME(to->drivername, from->drivername);
+	CORR_VALCPY(fetch_max);
+	CORR_VALCPY(unknown_sizes);
+	CORR_VALCPY(max_varchar_size);
+	CORR_VALCPY(max_longvarchar_size);
+	CORR_VALCPY(debug);
+	CORR_VALCPY(commlog);
+	CORR_VALCPY(unique_index);
+	CORR_VALCPY(onlyread);		/* readonly is reserved on Digital C++
+								 * compiler */
+	CORR_VALCPY(use_declarefetch);
+	CORR_VALCPY(text_as_longvarchar);
+	CORR_VALCPY(unknowns_as_longvarchar);
+	CORR_VALCPY(bools_as_char);
+	CORR_VALCPY(lie);
+	CORR_VALCPY(parse);
+	CORR_STRCPY(extra_systable_prefixes);
+	CORR_STRCPY(protocol);
+	NAME_TO_NAME(to->conn_settings, from->conn_settings);
+
+	mylog("copy_globals driver=%s\n", SAFE_NAME(to->drivername));
+}
+
+void	finalize_globals(GLOBAL_VALUES *glbv)
+{
+	NULL_THE_NAME(glbv->drivername);
+	NULL_THE_NAME(glbv->conn_settings);
+}
+
+#undef	CORR_STRCPY
+#undef	CORR_VALCPY
+#define	CORR_STRCPY(item)	strncpy_null(ci->item, sci->item, sizeof(ci->item))
+#define	CORR_VALCPY(item)	(ci->item = sci->item)
+
+void
+CC_copy_conninfo(ConnInfo *ci, const ConnInfo *sci)
+{
+	memset(ci, 0,sizeof(ConnInfo));
+
+	CORR_STRCPY(dsn);
+	CORR_STRCPY(desc);
+	CORR_STRCPY(drivername);
+	CORR_STRCPY(server);
+	CORR_STRCPY(database);
+	CORR_STRCPY(username);
+	NAME_TO_NAME(ci->password, sci->password);
+	CORR_STRCPY(port);
+	CORR_STRCPY(sslmode);
+	CORR_STRCPY(onlyread);
+	CORR_STRCPY(fake_oid_index);
+	CORR_STRCPY(show_oid_column);
+	CORR_STRCPY(row_versioning);
+	CORR_STRCPY(show_system_tables);
+	CORR_STRCPY(translation_dll);
+	CORR_STRCPY(translation_option);
+	CORR_VALCPY(password_required);
+	NAME_TO_NAME(ci->conn_settings, sci->conn_settings);
+	CORR_VALCPY(allow_keyset);
+	CORR_VALCPY(updatable_cursors);
+	CORR_VALCPY(lf_conversion);
+	CORR_VALCPY(true_is_minus1);
+	CORR_VALCPY(int8_as);
+	CORR_VALCPY(bytea_as_longvarbinary);
+	CORR_VALCPY(use_server_side_prepare);
+	CORR_VALCPY(lower_case_identifier);
+	CORR_VALCPY(rollback_on_error);
+	CORR_VALCPY(force_abbrev_connstr);
+	CORR_VALCPY(bde_environment);
+	CORR_VALCPY(fake_mss);
+	CORR_VALCPY(cvt_null_date_string);
+	CORR_VALCPY(accessible_only);
+	CORR_VALCPY(ignore_round_trip_time);
+	CORR_VALCPY(disable_keepalive);
+	CORR_VALCPY(gssauth_use_gssapi);
+	CORR_VALCPY(extra_opts);
+	CORR_VALCPY(keepalive_idle);
+	CORR_VALCPY(keepalive_interval);
+#ifdef	_HANDLE_ENLIST_IN_DTC_
+	CORR_VALCPY(xa_opt);
+#endif
+	copy_globals(&(ci->drivers), &(sci->drivers));	/* moved from driver's option */
+}
+#undef	CORR_STRCPY
+#undef	CORR_VALCPY
