@@ -10,7 +10,6 @@
 #define __DLG_SPECIFIC_H__
 
 #include "psqlodbc.h"
-#include "connection.h"
 
 #ifdef WIN32
 #include  <windowsx.h>
@@ -165,6 +164,8 @@ extern "C" {
 #define ABBR_KEEPALIVETIME		"D1"
 #define INI_KEEPALIVEINTERVAL		"KeepaliveInterval"
 #define ABBR_KEEPALIVEINTERVAL		"D2"
+#define INI_PQOPT			"pqopt"
+#define ABBR_PQOPT			"D5"
 #define INI_DTCLOG			"Dtclog"
 /* "PreferLibpq", abbreviated "D4", used to mean whether to prefer libpq.
  * libpq is now required
@@ -248,7 +249,7 @@ extern "C" {
 
 #define DEFAULT_CANCELASFREESTMT		0
 
-#define DEFAULT_EXTRASYSTABLEPREFIXES	"dd_"
+#define DEFAULT_EXTRASYSTABLEPREFIXES	""
 
 #define DEFAULT_TRUEISMINUS1		0
 #define DEFAULT_UPDATABLECURSORS	1
@@ -258,7 +259,7 @@ extern "C" {
 #define DEFAULT_LFCONVERSION		0
 #endif	/* WIN32 */
 #define DEFAULT_INT8AS			0
-#define DEFAULT_BYTEAASLONGVARBINARY	0
+#define DEFAULT_BYTEAASLONGVARBINARY	1
 #define DEFAULT_USESERVERSIDEPREPARE	1
 #define DEFAULT_LOWERCASEIDENTIFIER	0
 #define DEFAULT_SSLMODE			SSLMODE_DISABLE
@@ -268,6 +269,10 @@ extern "C" {
 #define DEFAULT_XAOPT			1
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
 
+/*	for CC_DSN_info */
+#define CONN_DONT_OVERWRITE		0
+#define CONN_OVERWRITE			1
+
 /*	prototypes */
 void		getCommonDefaults(const char *section, const char *filename, ConnInfo *ci);
 
