@@ -411,7 +411,7 @@ PGAPI_GetConnectAttr(HDBC ConnectionHandle,
 			*((SQLINTEGER *) Value) = SQL_FALSE;
 			break;
 		case SQL_ATTR_CONNECTION_DEAD:
-			*((SQLUINTEGER *) Value) = (conn->status == CONN_NOT_CONNECTED || conn->status == CONN_DOWN);
+			*((SQLUINTEGER *) Value) = CC_not_connected(conn);
 			break;
 		case SQL_ATTR_CONNECTION_TIMEOUT:
 			*((SQLUINTEGER *) Value) = 0;
@@ -1666,9 +1666,9 @@ PGAPI_SetConnectAttr(HDBC ConnectionHandle,
 			break;
 		case SQL_ATTR_PGOPT_DEBUG:
 			newValue = CAST_UPTR(SQLCHAR, Value);
-			if (newValue > 0 && conn->connInfo.drivers.debug <= 0)
+			if (newValue > 0)
 			{
-				logs_on_off(-1, 0, 0);
+				logs_on_off(-1, conn->connInfo.drivers.debug, 0);
 				conn->connInfo.drivers.debug = newValue;
 				logs_on_off(1, conn->connInfo.drivers.debug, 0);
 				mylog("debug => %d\n", conn->connInfo.drivers.debug);
@@ -1933,10 +1933,6 @@ inolog("set ard=%p\n", stmt->ard);
 	return ret;
 }
 
-#define	CALC_BOOKMARK_ADDR(book, offset, bind_size, index) \
-	(book->buffer + offset + \
-	(bind_size > 0 ? bind_size : (SQL_C_VARBOOKMARK == book->returntype ? book->buflen : sizeof(UInt4))) * index)
-
 /*	SQL_NEED_DATA callback for PGAPI_BulkOperations */
 typedef struct
 {
@@ -1954,12 +1950,12 @@ RETCODE	bulk_ope_callback(RETCODE retcode, void *para)
 	CSTR func = "bulk_ope_callback";
 	RETCODE	ret = retcode;
 	bop_cdata *s = (bop_cdata *) para;
-	SQLULEN		offset, global_idx;
-	SQLUINTEGER	bind_size;
+	SQLULEN		global_idx;
 	ConnectionClass	*conn;
 	QResultClass	*res;
 	IRDFields	*irdflds;
 	BindInfoClass	*bookmark;
+	PG_BM		pg_bm;
 
 	if (s->need_data_callback)
 	{
@@ -1973,15 +1969,14 @@ RETCODE	bulk_ope_callback(RETCODE retcode, void *para)
 	}
 	s->need_data_callback = FALSE;
 	bookmark = s->opts->bookmark;
-	offset = s->opts->row_offset_ptr ? *(s->opts->row_offset_ptr) : 0;
-	bind_size = s->opts->bind_size;
+	res = SC_get_Curres(s->stmt);
 	for (; SQL_ERROR != ret && s->idx < s->opts->size_of_rowset; s->idx++)
 	{
 		if (SQL_ADD != s->operation)
 		{
-			UInt4 tmp;
-			memcpy(&tmp, CALC_BOOKMARK_ADDR(bookmark, offset, bind_size, s->idx), sizeof(UInt4));
-			global_idx = SC_resolve_bookmark(tmp);
+			pg_bm = SC_Resolve_bookmark(s->opts, s->idx);
+			QR_get_last_bookmark(res, s->idx, &pg_bm.keys);
+			global_idx = pg_bm.index;
 		}
 		/* Note opts->row_operation_ptr is ignored */
 		switch (s->operation)
@@ -1990,13 +1985,10 @@ RETCODE	bulk_ope_callback(RETCODE retcode, void *para)
 				ret = SC_pos_add(s->stmt, (UWORD) s->idx);
 				break;
 			case SQL_UPDATE_BY_BOOKMARK:
-				ret = SC_pos_update(s->stmt, (UWORD) s->idx, global_idx);
+				ret = SC_pos_update(s->stmt, (UWORD) s->idx, global_idx, &(pg_bm.keys));
 				break;
 			case SQL_DELETE_BY_BOOKMARK:
-				ret = SC_pos_delete(s->stmt, (UWORD) s->idx, global_idx);
-				break;
-			case SQL_FETCH_BY_BOOKMARK:
-				ret = SC_pos_refresh(s->stmt, (UWORD) s->idx, global_idx);
+				ret = SC_pos_delete(s->stmt, (UWORD) s->idx, global_idx, &(pg_bm.keys));
 				break;
 		}
 		if (SQL_NEED_DATA == ret)
@@ -2022,7 +2014,7 @@ RETCODE	bulk_ope_callback(RETCODE retcode, void *para)
 	if (irdflds->rowsFetched)
 		*(irdflds->rowsFetched) = s->processed;
 
-	if (res = SC_get_Curres(s->stmt), res)
+	if (res)
 		res->recent_processed_row_count = s->stmt->diag_row_count = s->processed;
 	return ret;
 }
@@ -2059,8 +2051,13 @@ PGAPI_BulkOperations(HSTMT hstmt, SQLSMALLINT operationX)
 	}
 
 	/* StartRollbackState(s.stmt); */
-	s.need_data_callback = FALSE;
-	ret = bulk_ope_callback(SQL_SUCCESS, &s);
+	if (SQL_FETCH_BY_BOOKMARK == operationX)
+		ret = SC_fetch_by_bookmark(s.stmt);
+	else
+	{
+		s.need_data_callback = FALSE;
+		ret = bulk_ope_callback(SQL_SUCCESS, &s);
+	}
 	if (s.stmt->internal)
 		ret = DiscardStatementSvp(s.stmt, ret, FALSE);
 	return ret;
