@@ -1,4 +1,4 @@
-/*-------
+/*------
  * Module:			statement.c
  *
  * Description:		This module contains functions related to creating
@@ -170,7 +170,8 @@ static const struct
 };
 
 static QResultClass *libpq_bind_and_exec(StatementClass *stmt);
-static void SC_set_errorinfo(StatementClass *self, QResultClass *res);
+static void SC_set_errorinfo(StatementClass *self, QResultClass *res, int errkind);
+static void SC_set_error_if_not_set(StatementClass *self, int errornumber, const char *errmsg, const char *func);
 
 
 RETCODE		SQL_API
@@ -1609,7 +1610,7 @@ inolog("%s statement=%p res=%x ommitted=0\n", func, self, res);
 			(self->currTuple)++;	/* all is well */
 		else
 		{
-			SC_set_errorinfo(self, res);
+			SC_set_errorinfo(self, res, 1);
 			return SQL_ERROR;
 		}
 	}
@@ -1887,8 +1888,10 @@ SC_execute(StatementClass *self)
 		if (!res)
 		{
 			if (SC_get_errornumber(self) <= 0)
+			{
 				SC_set_error(self, STMT_NO_RESPONSE, "Could not receive the response, communication down ??", func);
-			CC_on_abort(conn, CONN_DEAD);
+				CC_on_abort(conn, CONN_DEAD);
+			}
 			goto cleanup;
 		}
 	}
@@ -2008,7 +2011,7 @@ SC_execute(StatementClass *self)
 		else if (was_nonfatal)
 			SC_set_errornumber(self, STMT_INFO_ONLY);
 		else
-			SC_set_errorinfo(self, res);
+			SC_set_errorinfo(self, res, 0);
 		/* set cursor before the first tuple in the list */
 		self->currTuple = -1;
 		SC_set_current_col(self, -1);
@@ -2339,6 +2342,11 @@ RequestStart(StatementClass *stmt, ConnectionClass *conn, const char *func)
 	if (conn->asdum)
 		CALL_IsolateDtcConn(conn, TRUE);
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
+	if (NULL == conn->pqconn)
+        {
+		SC_set_error(stmt, STMT_COMMUNICATION_ERROR, "The connection has been lost", __FUNCTION__);
+		return SQL_ERROR;
+	}
 	if (SC_accessed_db(stmt))
 		return TRUE;
 	if (SQL_ERROR == SetStatementSvp(stmt))
@@ -2346,7 +2354,7 @@ RequestStart(StatementClass *stmt, ConnectionClass *conn, const char *func)
 		char	emsg[128];
 
 		snprintf(emsg, sizeof(emsg), "internal savepoint error in %s", func);
-		SC_set_error(stmt, STMT_INTERNAL_ERROR, emsg, func);
+		SC_set_error_if_not_set(stmt, STMT_INTERNAL_ERROR, emsg, func);
 		return FALSE;
 	}
 
@@ -2390,7 +2398,7 @@ libpq_bind_and_exec(StatementClass *stmt)
 	{
 		if (SQL_ERROR == SetStatementSvp(stmt))
 		{
-			SC_set_error(stmt, STMT_INTERNAL_ERROR, "internal savepoint error in build_libpq_bind_params", func);
+			SC_set_error_if_not_set(stmt, STMT_INTERNAL_ERROR, "internal savepoint error in build_libpq_bind_params", func);
 			return NULL;
 		}
 	}
@@ -2427,11 +2435,12 @@ libpq_bind_and_exec(StatementClass *stmt)
 
 		if (!stmt->processed_statements)
 		{
-			if (prepareParametersNoDesc(stmt, FALSE) == SQL_ERROR)
+			if (prepareParametersNoDesc(stmt, FALSE, EXEC_PARAM_CAST) == SQL_ERROR)
 				goto cleanup;
 		}
 
 		pstmt = stmt->processed_statements;
+		mylog("%s execParams query=%s nParams=%d\n", __FUNCTION__, pstmt->query, nParams);
 		pgres = PQexecParams(conn->pqconn,
 							 pstmt->query,
 							 nParams,
@@ -2452,9 +2461,10 @@ libpq_bind_and_exec(StatementClass *stmt)
 		}
 
 		/* prepareParameters() set plan name, so don't fetch this earlier */
-		plan_name = stmt->plan_name ? stmt->plan_name : "";
+		plan_name = stmt->plan_name ? stmt->plan_name : NULL_STRING;
 
 		/* already prepared */
+		mylog("%s execPrepared plan=%s nParams=%d\n", __FUNCTION__, plan_name, nParams);
 		pgres = PQexecPrepared(conn->pqconn,
 							   plan_name, 	/* portal name == plan name */
 							   nParams,
@@ -2607,7 +2617,11 @@ ParseWithLibpq(StatementClass *stmt, const char *plan_name,
 			if (pidx > end_pidx)
 				break;
 			else if (pidx < end_pidx)
+			{
+				if (0 == num_params)
+					sta_pidx = pidx;
 				num_params++;
+			}
 			else
 			{
 				num_params++;
@@ -2922,9 +2936,13 @@ BOOL	SC_AcceptedCancelRequest(const StatementClass *self)
 static void
 SC_set_error_if_not_set(StatementClass *self, int errornumber, const char *errmsg, const char *func)
 {
-	if (SC_get_errormsg(self) <= 0)
+	int	errnum = SC_get_errornumber(self);
+
+	if (errnum <= 0)
 	{
-		if (!SC_get_errormsg(self))
+		const char *emsg = SC_get_errormsg(self);
+
+		if (emsg && 0 == errnum)
 			SC_set_errornumber(self, errornumber);
 		else
 			SC_set_error(self, errornumber, errmsg, func);
@@ -2932,7 +2950,7 @@ SC_set_error_if_not_set(StatementClass *self, int errornumber, const char *errms
 }
 
 static void
-SC_set_errorinfo(StatementClass *self, QResultClass *res)
+SC_set_errorinfo(StatementClass *self, QResultClass *res, int errkind)
 {
 	ConnectionClass *conn = SC_get_conn(self);
 
@@ -2954,7 +2972,15 @@ SC_set_errorinfo(StatementClass *self, QResultClass *res)
 			SC_set_error_if_not_set(self, STMT_INTERNAL_ERROR, "Internal error fetching next row", __FUNCTION__);
 			break;
 		default:
-			SC_set_error_if_not_set(self, STMT_EXEC_ERROR, "Error fetching next row", __FUNCTION__);
+			switch (errkind)
+			{
+				case 1:
+					SC_set_error_if_not_set(self, STMT_EXEC_ERROR, "Error while fetching the next result", __FUNCTION__);
+					break;
+				default:
+					SC_set_error_if_not_set(self, STMT_EXEC_ERROR, "Error while executing the query", __FUNCTION__);
+					break;
+			}
 			break;
 	}
 }
